if ( !SMTPSwitchReborn ) var SMTPSwitchReborn = {};

// JSCTSABT: Just Some Checks, This Should Always Be True

/**
 * (class) Simplifies manipulation of a local file between TB versions
 *
 * nsILocalFile was merged with the interface of nsIFile in Gecko 14
 * Much of the documentation has not been updated to reflect this change.
 * - TB <= v13.*
 *   (nsILocalFile)
 * - TB >= v14.0a1
 *   (nsIFile)
 *
 * @return (object) file object
 */
SMTPSwitchReborn.VirtualLocalFile = function ()
{
	var fileObj = null;

	if ( SMTPSwitchReborn.isTBVersionEqualOrHigherThan("14.0a1") ) { // if current TB version uses gecko engine v14.0
		fileObj = Components.classes["@mozilla.org/file/local;1"]
					.createInstance(Components.interfaces.nsIFile);
	}
	else {
		fileObj = Components.classes["@mozilla.org/file/local;1"]
					.createInstance(Components.interfaces.nsILocalFile);
	}

	return fileObj;
};


/**********
 * Add-on constants
 */

SMTPSwitchReborn.PR_RDONLY		= 0x01;
SMTPSwitchReborn.PR_WRONLY		= 0x02;
SMTPSwitchReborn.PR_RDWR		= 0x04;
SMTPSwitchReborn.PR_CREATE_FILE	= 0x08;
SMTPSwitchReborn.PR_APPEND		= 0x10;
SMTPSwitchReborn.PR_TRUNCATE	= 0x20;
SMTPSwitchReborn.PR_SYNC		= 0x40;
SMTPSwitchReborn.PR_EXCL		= 0x80;


/**
 * (class) Allows to use error console embedded in Mozilla apps
 */
SMTPSwitchReborn.Console = function ()
{
	this.console = Components.classes["@mozilla.org/consoleservice;1"]
					.getService(Components.interfaces.nsIConsoleService);

	/**
	 * Adds 0-padding of given length to given number
	 *
	 * @param number (integer) number to format
	 * @param formatLength (integer) length of 0-padding to add
	 * @return (string) formatted number as string
	 */
	this.formatNumber = function (number, formatLength) {
		var paddingStr = "";
		var nmbrStr = "";
		var paddingLength = formatLength - number.toString().length;

		for ( var idx = 0 ; idx < paddingLength ; idx++ ) {
			paddingStr += "0";
		}
		nmbrStr = paddingStr + number.toString();

		return nmbrStr;
	}

	/**
	 * Formats date
	 *
	 * @param dt (Date object) date/time to format
	 * @return (string) formatted date as string
	 */
	this.formatDate = function (dt) {
		return dt.getFullYear() + "-" + this.formatNumber(dt.getMonth() + 1, 2) + "-" + this.formatNumber(dt.getDate(), 2);
	}

	/**
	 * Formats time
	 *
	 * @param dt (Date object) date/time to format
	 * @return (string) formatted time as string
	 */
	this.formatTime = function (dt) {
		return this.formatNumber(dt.getHours(), 2) + ":" + this.formatNumber(dt.getMinutes(), 2) + ":" + this.formatNumber(dt.getSeconds(), 2) + "." + this.formatNumber(dt.getMilliseconds(), 3);
	}

	/**
	 *
	 */
	this.logInformation = function (source, message, forceFlag) {
		forceFlag = ( typeof(forceFlag) !== "undefined" ) ? forceFlag : false;

		if ( SMTPSwitchReborn.debugMode || forceFlag ) {
			var dt = new Date();
			var dateTime = this.formatDate(dt) + " " + this.formatTime(dt);

			this.console.logStringMessage(dateTime + " [" + SMTPSwitchReborn.addOnName + "." + source + "]: " + message);
			SMTPSwitchReborn.writeMessageToFile("Information;" + dateTime + ";" + source + ";" + message);
		}
	}

	/**
	 *
	 */
	this.logError = function (source, message, noForceFlag) {
		noForceFlag = ( typeof(noForceFlag) !== "undefined" ) ? noForceFlag : true;

		var error = Components.classes["@mozilla.org/scripterror;1"]
						.createInstance(Components.interfaces.nsIScriptError);
		var dt = new Date();
		var dateTime = this.formatDate(dt) + " " + this.formatTime(dt);

		error.init(dateTime + " [" + SMTPSwitchReborn.addOnName + "." + source + "]: " + message, "", null, null, null, Components.interfaces.nsIScriptError.errorFlag, null);

		this.console.logMessage(error);

		if ( noForceFlag ) {
			SMTPSwitchReborn.writeMessageToFile("Error;" + dateTime + ";" + source + ";" + message);
		}
	}

	/**
	 *
	 */
	this.logWarning = function (source, message) {
		var warning = Components.classes["@mozilla.org/scripterror;1"]
						.createInstance(Components.interfaces.nsIScriptError);
		var dt = new Date();
		var dateTime = this.formatDate(dt) + " " + this.formatTime(dt);

		warning.init(dateTime + " [" + SMTPSwitchReborn.addOnName + "." + source + "]: " + message, "", null, null, null, Components.interfaces.nsIScriptError.warningFlag, null);

		this.console.logMessage(warning);
		SMTPSwitchReborn.writeMessageToFile("Warning;" + dateTime + ";" + source + ";" + message);
	}
};


/**
 * (class) Simplifies manipulation of lists between TB versions
 *
 * Supports nsISupportsArray (TB <= v19.*) and nsIArray (TB >= v20.0a1)
 *
 * @param obj (object) object to virtualize
 * @param uuid (nsIIDRef) type of object contained in list
 */
SMTPSwitchReborn.VirtualArray = function (obj, uuid)
{
	// attributes
	this.realObj = obj;
	this.uuid = uuid;

	// methods
	this.count = function () {
		var cnt = -1;

		if ( this.realObj != null ) {
			if ( this.realObj.toString() === "[xpconnect wrapped nsISupportsArray]" ) {
				cnt = this.realObj.Count();
			}
			else if ( this.realObj.toString() === "[xpconnect wrapped nsIArray]" ) {
				cnt = this.realObj.length;
			}
			else {
				throw this.realObj.toString() + " is not a handled class"; // throws an exception
			}
		}

		return cnt;
	}

	this.queryAt = function (index) {
		var elt = null;

		if ( this.realObj != null ) {
			if ( this.realObj.toString() === "[xpconnect wrapped nsISupportsArray]" ) {
				elt = this.realObj.QueryElementAt(index, this.uuid);
			}
			else if ( this.realObj.toString() === "[xpconnect wrapped nsIArray]" ) {
				elt = this.realObj.queryElementAt(index, this.uuid);
			}
			else {
				throw this.realObj.toString() + " is not a handled class"; // throws an exception
			}
		}

		return elt;
	}
};


/**
 * (class) Holds information about detection rule
 *
 * @param objStartIP (object/string) object / start ip
 * @param endIP (string) end ip
 * @param smtpId (string) smtp server id
 * @param on (boolean) true/false if detection rule is enabled/disabled
 */
SMTPSwitchReborn.DetectionRule = function (objStartIP, endIP, smtpId, on)
{
	if ( typeof(objStartIP) === 'object' ) { // if first parameter is an object (in this case this should be the one and only parameter)
		this.startIP = objStartIP.startIP;
		this.endIP = objStartIP.endIP;
		this.smtpId = objStartIP.smtpId;
		this.on = objStartIP.on;
	}
	else { // if first parameter is not an object (in this case all other parameters should be defined)
		this.startIP = objStartIP;
		this.endIP = endIP;
		this.smtpId = smtpId;
		this.on = on;
	}
};


/**********
 * Add-on variables
 */

SMTPSwitchReborn.addOnName = "SMTPSwitchReborn";
SMTPSwitchReborn.addOnId = "SMTPSwitchReborn@gmail.com";
SMTPSwitchReborn.temporaryRoot = "SSR";
SMTPSwitchReborn.exportSettingsMode = "PREFS"; // PREFS or INI
SMTPSwitchReborn.prefs = null;

SMTPSwitchReborn.localConsole = new SMTPSwitchReborn.Console();
SMTPSwitchReborn.stringBundle = null;

SMTPSwitchReborn.autoDetectionAtStart = false;
SMTPSwitchReborn.autoDetectionAtCompose = false;
SMTPSwitchReborn.detectionRules = "";
SMTPSwitchReborn.displayServerAddresses = false;
SMTPSwitchReborn.debugMode = false;
SMTPSwitchReborn.debugModeToFile = false;
SMTPSwitchReborn.debugFileName = "";


/**********
 * Common methods
 */

/**
 * Finishes add-on execution
 */
SMTPSwitchReborn.shutdown = function ()
{
	this.prefs.removeObserver("", this);
};

/**
 * Observes modifications of settings
 *
 * @param prefBranch (nsIPrefBranch) observed preferences branch
 * @param topic (string) fired event
 * @param data (string) name of changed preference
 */
SMTPSwitchReborn.observe = function (prefBranch, topic, data)
{
	if ( topic == "nsPref:changed" ) {
		switch ( data ) {
			case "autoDetectionAtStart":
				this.autoDetectionAtStart = this.prefs.getBoolPref("autoDetectionAtStart");
				this.detectSMTPServerAuto();
				break;
			case "autoDetectionAtCompose": this.autoDetectionAtCompose = this.prefs.getBoolPref("autoDetectionAtCompose"); break;
			case "detectionRules":
				this.detectionRules = this.prefs.getCharPref("detectionRules");
				this.detectSMTPServerAuto();
				break;
			case "displayServerAddresses": this.displayServerAddresses = this.prefs.getBoolPref("displayServerAddresses"); break;
			case "debugMode": this.debugMode = this.prefs.getBoolPref("debugMode"); break;
			case "debugModeToFile": this.debugModeToFile = this.prefs.getBoolPref("debugModeToFile"); break;
			case "debugFileName": this.debugFileName = unescape(this.prefs.getCharPref("debugFileName")); break;
		}
	}
};

/**
 * Gets Thunderbird version
 *
 * @param comparedVersion (string) version to compare current version to
 * @return (boolean) true/false if Thunderbird version is equal or higher/lower than given version
 */
SMTPSwitchReborn.isTBVersionEqualOrHigherThan = function (comparedVersion)
{
	var version;
	var versionChecker;

	version = ( "@mozilla.org/xre/app-info;1" in Components.classes ) ?
		Components.classes["@mozilla.org/xre/app-info;1"]
			.getService(Components.interfaces.nsIXULAppInfo)
			.version
		:
		Components.classes["@mozilla.org/preferences-service;1"]
			.getService(Components.interfaces.nsIPrefBranch)
			.getCharPref("app.version");

	versionChecker = Components.classes["@mozilla.org/xpcom/version-comparator;1"]
						.getService(Components.interfaces.nsIVersionComparator);

	if ( versionChecker.compare(version, comparedVersion) >= 0 ) {
		return true;
	}

	return false;
};

/**
 * Displays given message in a message dialog
 *
 * @param message (string) message to be displayed
 */
SMTPSwitchReborn.displayMessageDialog = function (message)
{
	var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
						.getService(Components.interfaces.nsIPromptService);

	promptService.confirmEx(window, this.addOnName, message, promptService.BUTTON_TITLE_OK * promptService.BUTTON_POS_0, "", "", null, "", {value: false});
};

/**
 * Displays given message in an alert dialog
 *
 * @param message (string) message to be displayed
 */
SMTPSwitchReborn.displayAlertDialog = function (message)
{
	var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
						.getService(Components.interfaces.nsIPromptService);

	promptService.alert(null, this.addOnName, message);
};

/**
 * Displays given message in a dialog and ask for confirmation
 *
 * @param message (string) message to be displayed
 * @return (boolean) true/false if user has/has not confirmed
 */
SMTPSwitchReborn.displayConfirmDialog = function (message)
{
	var promptService = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
						.getService(Components.interfaces.nsIPromptService);

	return promptService.confirm(null, this.addOnName, message);
};

/**
 * Displays given message in a popup dialog
 *
 * @param message (string) message to be displayed
 */
SMTPSwitchReborn.displayPopUpMessage = function (message)
{
	try {
		var alertService = Components.classes["@mozilla.org/alerts-service;1"]
							.getService(Components.interfaces.nsIAlertsService);

		alertService.showAlertNotification("chrome://SMTPSwitchReborn/skin/icon.png", this.addOnName, message, false, "", null, "");
	}
	catch ( err ) {
		var windowWatcher = Components.classes["@mozilla.org/embedcomp/window-watcher;1"]
					.getService(Components.interfaces.nsIWindowWatcher)
					.openWindow(null, "chrome://global/content/alerts/alert.xul", "_blank", "chrome,titlebar=no,popup=yes", null);

		windowWatcher.arguments = ["chrome://SMTPSwitchReborn/skin/icon.png", this.addOnName, message, false, ""];
	}
};

/**
 * Loads and displays given page
 *
 * @param pageUrl (string) page URL to load and display
 */
SMTPSwitchReborn.displayPage = function (pageUrl)
{
	var mail3PaneWindow = Components.classes["@mozilla.org/appshell/window-mediator;1"]
							.getService(Components.interfaces.nsIWindowMediator)
							.getMostRecentWindow("mail:3pane");

	if ( mail3PaneWindow != null ) {
		var tabmail = mail3PaneWindow.document.getElementById("tabmail");

		if ( tabmail != null ) { // if application supports tabs (TB >= v3.*)
			tabmail.openTab("contentTab", {contentPage: pageUrl});
		}
		else { // if application does not support tabs (TB < v3.*)
			window.opener.openDialog(pageUrl, "_blank", "chrome,resizable=yes,height=800,width=1000");
		}
	}
};

// for TB v3.* there is a bug in native functions JSON.stringify() and JSON.parse() (checked from Shredder v3.0.6 to Lanikai 3.1.21):
// when using an object and a replacer function, properties values of the object are not returned correctly:
// they stay unchanged, even if modified into replacer function

/**
 * Turns given object into a JSON string using JSON engine
 *
 * Aimed to replace native function JSON.stringify() for TB v3.*:
 * First the names are escaped (using escape()), then the whole object is stringified using native JSON function
 *
 * @param obj (object) object to stringify
 * @param replacer (function) function to apply
 * @param keysArray (array of strings) list of properties to apply function to
 * @return (string) JSON
 */
SMTPSwitchReborn.JSONstringify = function (obj, replacer, keysArray)
{
	if ( typeof(obj) !== 'object' ) {
		throw "Error in function JSONstringify(): Argument is not a valid object but a " + typeof(obj);
	}

	if ( (replacer != null ) && (obj != null) ) { // if replacement function is defined and given object is not null
		if ( typeof(obj.length) === 'number' ) { // if given object is an array
			for ( var idx = 0 ; idx < obj.length ; idx++ ) { // for each element
				for ( var propty in obj[idx] ) { // for each property of current element
					if ( keysArray.indexOf(propty) != -1 ) { // if property is one of allowed keys
						obj[idx][propty] = replacer(propty, obj[idx][propty]); // applies replacement function
					}
				}
			}
		}
		else { // if given object is a single object
			for ( var propty in obj ) { // for each property of current element
				if ( keysArray.indexOf(propty) != -1 ) { // if property is one of allowed keys
					obj[propty] = replacer(propty, obj[propty]); // applies replacement function
				}
			}
		}
	}

	return JSON.stringify(obj);
};

/**
 * Gets a JSON object from given string considered as a JSON string
 *
 * Aimed to replace native function JSON.parse() for TB v3.*:
 * First the string is parsed using native JSON function, then names are unescaped (using unescape())
 *
 * @param str (string) string to parse
 * @param replacer (function) function to apply
 * @param keysArray (array of strings) list of properties to apply function to
 * @return (object) resulting object
 */
SMTPSwitchReborn.JSONparse = function (str, replacer, keysArray)
{
	if ( typeof(str) !== 'string' ) {
		throw "Error in function JSONparse(): Argument is not a valid string but a " + typeof(str);
	}
	else if ( str === "" ) {
		throw "Error in function JSONparse(): Argument is an empty string";
	}

	var obj = JSON.parse(str);

	if ( (replacer != null ) && (obj != null) ) { // if replacement function is defined and JSON-parsed object is not null
		if ( typeof(obj.length) === 'number' ) { // if JSON-parsed object is an array
			for ( var idx = 0 ; idx < obj.length ; idx++ ) { // for each element
				for ( var propty in obj[idx] ) { // for each property of current element
					if ( keysArray.indexOf(propty) != -1 ) { // if property is one of allowed keys
						obj[idx][propty] = replacer(propty, obj[idx][propty]); // applies replacement function
					}
				}
			}
		}
		else { // if JSON-parsed object is a single object
			for ( var propty in obj ) { // for each property of current element
				if ( keysArray.indexOf(propty) != -1 ) { // if property is one of allowed keys
					obj[propty] = replacer(propty, obj[propty]); // applies replacement function
				}
			}
		}
	}

	return obj;
};

/**
 * Applies unescape() function on string properties
 *
 * @param key (string) property key
 * @param value (string) property value
 * @return (string) modified property value
 */
SMTPSwitchReborn.preProcessLoadSettings = function (key, value)
{
	if ( typeof(value) === 'string' ) { // if property is a string
		value = unescape(value);
	}

	return value;
};

/**
 * Applies escape() function on string properties
 *
 * @param key (string) property key
 * @param value (string) property value
 * @return (string) modified property value
 */
SMTPSwitchReborn.preProcessSaveSettings = function (key, value)
{
	if ( typeof(value) === 'string' ) { // if property is a string
		value = escape(value);
	}

	return value;
};

/**
 * Writes given message to log file
 *
 * @param message (string) message to log
 */
SMTPSwitchReborn.writeMessageToFile = function (message)
{
	if ( SMTPSwitchReborn.debugMode && SMTPSwitchReborn.debugModeToFile && (SMTPSwitchReborn.debugFileName != "") ) {
		try {
			// check existence of debug log file
			var debugFile = SMTPSwitchReborn.VirtualLocalFile();
			debugFile.initWithPath(SMTPSwitchReborn.debugFileName);

			if ( !debugFile.exists() ) {
				debugFile.create(Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 0664);
			}

			// writing of message into debug log file
			var debugFileStream = Components.classes["@mozilla.org/network/file-output-stream;1"]
									.createInstance(Components.interfaces.nsIFileOutputStream);

			debugFileStream.init(debugFile, SMTPSwitchReborn.PR_WRONLY | SMTPSwitchReborn.PR_APPEND, 0664, 0);

			var converter = Components.classes["@mozilla.org/intl/converter-output-stream;1"]
							.createInstance(Components.interfaces.nsIConverterOutputStream);
			converter.init(debugFileStream, "UTF-8", 0, 0);
			converter.writeString(message + "\n");
			converter.close(); // closes debugFileStream too
		}
		catch ( err ) {
			SMTPSwitchReborn.debugModeToFile = false;

			SMTPSwitchReborn.localConsole.logError("writeMessageToFile", "Error while writing log message to debug file \"" + message + "\" (" + err + ")", false);
		}
	}
};

/**
 * Launchs SMTP server detection using detection rules
 */
SMTPSwitchReborn.launchSMTPServerDetection = function ()
{
	var dnsService = Components.classes["@mozilla.org/network/dns-service;1"]
						.createInstance(Components.interfaces.nsIDNSService);

	var dnsResolveThread = Components.classes["@mozilla.org/thread-manager;1"]
							.getService(Components.interfaces.nsIThreadManager).currentThread;

	this.localConsole.logInformation("launchSMTPServerDetection", "Retrieving IP addresses for local host '" + dnsService.myHostName + "'...");

	dnsService.asyncResolve(dnsService.myHostName, Components.interfaces.nsIDNSService.RESOLVE_CANONICAL_NAME | Components.interfaces.nsIDNSService.RESOLVE_DISABLE_IPV6, new SMTPSwitchReborn.DNSResolveObserver(dnsService.myHostName), dnsResolveThread);

	// process will be pursued in the DNSResolveObserver.onLookupComplete handler (finalizeSMTPServerDetection()) once DNS request will be completed
}

/**
 * Detects SMTP server using detection rules
 *
 * @param dnsRecords (array of nsIDNSRecord objects) DNS records returned by the DNS request
 */
SMTPSwitchReborn.finalizeSMTPServerDetection = function (dnsRecords)
{
	var reIPAddress = new RegExp("^(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$", "i");

	var ipAddress = "";

	var startIP = "";
	var endIP = "";
	var smtpId = "";

	var ipParts = null;
	var startIPParts = null;
	var endIPParts = null;

	var okFlag = true;
	var foundFlag = true;

	// extraction of detection rules list
	var detectionRulesArray = null;

	if ( (this.detectionRules == null) || (this.detectionRules == "") ) { // if detection rules list is empty
		detectionRulesArray = new Array();
	}
	else {
		detectionRulesArray = SMTPSwitchReborn.JSONparse(this.detectionRules, SMTPSwitchReborn.preProcessLoadSettings, new Array("startIP", "endIP", "smtpId"));
	}

	while ( dnsRecords.hasMore() ) {
		ipAddress = dnsRecords.getNextAddrAsString();

		this.localConsole.logInformation("finalizeSMTPServerDetection", "Checking " + ipAddress + "...");

		ipParts = ipAddress.match(reIPAddress);

		if ( ipParts != null ) {
			foundFlag = false;

			for ( var idx = 0 ; idx < detectionRulesArray.length && !foundFlag ; idx++ ) { // for each detection rule
				startIP = detectionRulesArray[idx].startIP;
				endIP = detectionRulesArray[idx].endIP;
				smtpId = detectionRulesArray[idx].smtpId;

				this.localConsole.logInformation("finalizeSMTPServerDetection", "Checking rule [" + startIP + ";" + endIP + "] => " + smtpId);

				startIPParts = startIP.match(reIPAddress);
				endIPParts = endIP.match(reIPAddress);

				if ( (startIPParts != null) && (endIPParts != null) ) {
					okFlag = true;

					for ( var jdx = 1 ; (jdx <= 4) && okFlag ; jdx++ ) {
						if ( (parseInt(ipParts[jdx]) < parseInt(startIPParts[jdx])) || (parseInt(ipParts[jdx]) > parseInt(endIPParts[jdx])) ) {
							okFlag = false;
						}
					}

					if ( okFlag ) {
						this.localConsole.logInformation("finalizeSMTPServerDetection", "IP address " + ipAddress + " validates rule [" + startIP + ";" + endIP + "] => " + smtpId);

						if ( detectionRulesArray[idx].on ) {
							foundFlag = true;
						}
						else {
							this.localConsole.logInformation("finalizeSMTPServerDetection", "Rule [" + startIP + ";" + endIP + "] => " + smtpId + " is disabled: skipped");
						}
					}
				}
				else {
					this.localConsole.logError("finalizeSMTPServerDetection", "Rule [" + startIP + ";" + endIP + "] => " + smtpId + " is invalid");
				}
			}

			if ( foundFlag ) {
				var smtpService = Components.classes["@mozilla.org/messengercompose/smtp;1"]
									.getService(Components.interfaces.nsISmtpService);

				if ( smtpId != smtpService.defaultServer.key ) {
					var newDefaultSMTPServer = smtpService.getServerByKey(smtpId);

					if ( newDefaultSMTPServer != null ) {
						smtpService.defaultServer = newDefaultSMTPServer;

						this.localConsole.logInformation("finalizeSMTPServerDetection", "Default SMTP server has been changed to " + newDefaultSMTPServer.displayname + " (" + newDefaultSMTPServer.key + ")");
					}
					else {
						this.localConsole.logError("finalizeSMTPServerDetection", "Default SMTP server has not been found (" + smtpId + ")");
					}
				}
				else {
					this.localConsole.logInformation("finalizeSMTPServerDetection", "Default SMTP server has not been changed (" + smtpId + ")");
				}
			}
		}
		else {
			this.localConsole.logInformation("finalizeSMTPServerDetection", ipAddress + " is not a valid IPv4 address", "warning");				
		}
	}
};

/**
 * Detects SMTP server automatically using detection rules
 */
SMTPSwitchReborn.detectSMTPServerAuto = function ()
{
	if ( this.autoDetectionAtStart ) {
		this.launchSMTPServerDetection();
	}
	else {
		this.localConsole.logInformation("detectSMTPServerAuto", "SMTP server auto detection is disabled");
	}
};


/**
 * (class) Manages behaviour when a DNS request has been completed
 *
 * @param hostName (string) host name
 */
SMTPSwitchReborn.DNSResolveObserver = function (hostName)
{
	this.onLookupComplete = function (request, records, status) {
		if ( Components.isSuccessCode(status) ) {
			SMTPSwitchReborn.finalizeSMTPServerDetection(records); // finalizes IP detection
		}
		else {
			SMTPSwitchReborn.localConsole.logError("DNSResolveObserver.onLookupComplete", "Error while retrieving IP addresses for local host '" + hostName + "'");
		}
	}
};