if ( !SMTPSwitchReborn ) var SMTPSwitchReborn = {};


/**
 * Reads add-on settings
 */
SMTPSwitchReborn.readSettings = function ()
{
	this.autoDetectionAtStart = this.prefs.getBoolPref("autoDetectionAtStart");
	this.autoDetectionAtCompose = this.prefs.getBoolPref("autoDetectionAtCompose");
	this.detectionRules = this.prefs.getCharPref("detectionRules");
	this.displayServerAddresses = this.prefs.getBoolPref("displayServerAddresses");
	this.debugMode = this.prefs.getBoolPref("debugMode");
	this.debugModeToFile = this.prefs.getBoolPref("debugModeToFile");
	this.debugFileName = unescape(this.prefs.getCharPref("debugFileName")); // decodes string
};

/**
 * Performs some cleaning when addon is updated
 */
SMTPSwitchReborn.cleanUpdate = function ()
{
	var addonVersionChecker = Components.classes["@mozilla.org/xpcom/version-comparator;1"]
								.getService(Components.interfaces.nsIVersionComparator);

	if ( this.prefs.prefHasUserValue("version") ) {
	}
};

/**
 * Initializes startup
 */
SMTPSwitchReborn.initializeOverLayStartup = function ()
{
	if ( this.isTBVersionEqualOrHigherThan("3.3a1pre") ) { // if current TB version uses gecko engine v2.0 (starting from TB v3.3a1pre)
		Components.utils.import("resource://gre/modules/AddonManager.jsm");
		AddonManager.getAddonByID(this.addOnId, function (addon) { // asks for addon id
			SMTPSwitchReborn.overlayStartup(addon.version);
		});
	}
	else {
		var extensionManager = Components.classes["@mozilla.org/extensions/manager;1"]
								.getService(Components.interfaces.nsIExtensionManager);
		this.overlayStartup(extensionManager.getItemForID(this.addOnId).version);
	}

	// for TB version >= 3.3a1pre process will be pursued in overlayStartup() as soon as addon manager has loaded information (AddonManager.getAddonByID())
	// for TB version < 3.3a1pre process is pursued instantly
};

/**
 * Initializes add-on when starting Thunderbird composing session
 *
 * @param currentVersion (string) addon version
 */
SMTPSwitchReborn.overlayStartup = function (currentVersion)
{
	// system info
	var runtimeInfo = Components.classes["@mozilla.org/xre/app-info;1"]  
					.getService(Components.interfaces.nsIXULRuntime); 
	var osVersion = runtimeInfo.OS;
	var x3264 = ( runtimeInfo.is64Bit ) ? "64bits" : "32bits";
	var procInfo = runtimeInfo.XPCOMABI;

	var tbVersion = ( "@mozilla.org/xre/app-info;1" in Components.classes ) ?
		Components.classes["@mozilla.org/xre/app-info;1"]
			.getService(Components.interfaces.nsIXULAppInfo)
			.version
		:
		Components.classes["@mozilla.org/preferences-service;1"]
			.getService(Components.interfaces.nsIPrefBranch)
			.getCharPref("app.version");

	this.localConsole.logInformation("initializeOverlayStartup", "OS: " + osVersion + " " + x3264 + " (" + procInfo + ")", true);
	this.localConsole.logInformation("initializeOverlayStartup", "Thunderbird: v" + tbVersion, true);
	this.localConsole.logInformation("initializeOverlayStartup", this.addOnName + ": v" + currentVersion, true);

	// startup
	var newVersion = 0;

	this.prefs = Components.classes["@mozilla.org/preferences-service;1"]
					.getService(Components.interfaces.nsIPrefService)
					.getBranch("extensions." + this.addOnName + ".");

	// nsIPrefBranch2 allows to observe for changes to preferences
	// During Gecko 13 development, nsIPrefBranch2 was deprecated, and its methods moved to nsIPrefBranch
	// Calling .QueryInterface(Components.interfaces.nsIPrefBranch2) is no longer required, although it still works
	if ( this.isTBVersionEqualOrHigherThan("13.0a1") ) { // if current TB version uses gecko engine v13.0
		this.prefs.QueryInterface(Components.interfaces.nsIPrefBranch);
	}
	else {
		this.prefs.QueryInterface(Components.interfaces.nsIPrefBranch2);
	}

	this.prefs.addObserver("", this, false); // receives notifications if any preference changes

	this.cleanUpdate(); // makes some cleaning when updating addon version

	// management of version update
	if ( !this.prefs.prefHasUserValue("version") ) { // if version is undefined (= if this is first run)
		newVersion = 2;
	}
	else if ( currentVersion != this.prefs.getCharPref("version") ) { // ... or if version is different from current version
		newVersion = 1;
	}

	if ( newVersion > 0 ) { // if version has changed
		// add of addon version
		this.prefs.setIntPref("newVersion", newVersion);
		this.prefs.setCharPref("version", currentVersion);

		// display of help page
		this.displayPopUpMessage(this.addOnName + " v" + currentVersion + " has been installed");
		//this.displayPage("chrome://" + this.addOnName + "/content/help/help.htm");
	}

	this.readSettings(); // reads settings

	//
	this.localConsole.logInformation("overlayStartup", "Initialization has been completed", true);

	//
	this.detectSMTPServerAuto();
};


window.addEventListener("load", function() { SMTPSwitchReborn.initializeOverLayStartup(); }, false);
window.addEventListener("unload", function() { SMTPSwitchReborn.shutdown(); }, false);
