if ( !SMTPSwitchReborn ) var SMTPSwitchReborn = {};


/**********
 * Add-on variables
 */

// attributes are defined in SMTPSwitchReborn0.js

SMTPSwitchReborn.identityNameObj = {};


/**********
 * Command methods
 */

/**
 * Reads add-on settings
 */
SMTPSwitchReborn.readSettings = function ()
{
	this.autoDetectionAtStart = this.prefs.getBoolPref("autoDetectionAtStart");
	this.autoDetectionAtCompose = this.prefs.getBoolPref("autoDetectionAtCompose");
	this.detectionRules = this.prefs.getCharPref("detectionRules");
	this.displayServerAddresses = this.prefs.getBoolPref("displayServerAddresses");
	this.debugMode = this.prefs.getBoolPref("debugMode");
	this.debugModeToFile = this.prefs.getBoolPref("debugModeToFile");
	this.debugFileName = unescape(this.prefs.getCharPref("debugFileName")); // decodes string

	this.doNotSend = false;
};

/**
 * Initializes add-on when starting Thunderbird composing session
 */
SMTPSwitchReborn.composerOverlayStartup = function ()
{
	this.stringBundle = document.getElementById("smtpSwitchRebornStringBundle");

	this.prefs = Components.classes["@mozilla.org/preferences-service;1"]
					.getService(Components.interfaces.nsIPrefService)
					.getBranch("extensions." + this.addOnName + ".");

	if ( this.isTBVersionEqualOrHigherThan("13.0a1") ) { // if current TB version uses gecko engine v13.0
		this.prefs.QueryInterface(Components.interfaces.nsIPrefBranch);
	}
	else {
		this.prefs.QueryInterface(Components.interfaces.nsIPrefBranch2);
	}

	this.prefs.addObserver("", this, false); // receives notifications if any preference changes

	//
	this.identityNameObj = this.buildIdentityNameTable();

	this.readSettings(); // reads settings

	this.initializeSendButton();

	//
	this.localConsole.logInformation("composerOverlayStartup", "Initialization has been completed", true);

	//
	if ( this.autoDetectionAtCompose ) {
		this.launchSMTPServerDetection();
	}
};

/**
 * Initializes options of native 'Send Mail' button
 */
SMTPSwitchReborn.initializeSendButton = function ()
{
	var sendButton = document.getElementById("button-send");

	if ( sendButton != null ) {
		var sendButtonMenuPopUp = null;

		sendButton.setAttribute('type', "menu-button");

		var xulElement = sendButton.firstChild; // initializes xul element to first child

		// search of any menupopup of native 'Send Mail' button
		while ( (xulElement != null) && (sendButtonMenuPopUp == null) ) { // while there is a next element
			if ( xulElement.nodeName === "menupopup" ) { // if element is a menupopup
				sendButtonMenuPopUp = xulElement; // stores reference to menupopup
			}

			xulElement = xulElement.nextSibling; // gets next element
		}

		if ( sendButtonMenuPopUp == null ) {
			sendButtonMenuPopUp = document.createElement("menupopup"); // creates a new menupopup
			sendButtonMenuPopUp.setAttribute('id', "smtpSwitchRebornMenuPopUp");
			sendButtonMenuPopUp.setAttribute('onpopupshowing', "SMTPSwitchReborn.updateSMTPServersList();");
			sendButtonMenuPopUp.setAttribute('onpopuphidden', "SMTPSwitchReborn.clearSMTPServersList();");

			sendButton.appendChild(sendButtonMenuPopUp); // adds menupopup to button
		}
		else {
			sendButtonMenuPopUp.setAttribute('id', "smtpSwitchRebornMenuPopUp");
			sendButtonMenuPopUp.setAttribute('onpopupshowing', "SMTPSwitchReborn.updateSMTPServersList();");
			sendButtonMenuPopUp.setAttribute('onpopuphidden', "SMTPSwitchReborn.clearSMTPServersList();");

			// insertion of separator
			var separatorMenuItem = document.createElement("menuseparator"); // creates a new menuitem
			separatorMenuItem.setAttribute('id', "smtpSwitchReborn-separator-0");

			sendButtonMenuPopUp.appendChild(separatorMenuItem);
		}
	}
};

/**
 * Creates table of accounts identity & name
 *
 * @return (object) identity/name table
 */
SMTPSwitchReborn.buildIdentityNameTable = function ()
{
	var identityNameObj = {};

	var accountManager = Components.classes["@mozilla.org/messenger/account-manager;1"]
							.getService(Components.interfaces.nsIMsgAccountManager);

	var accounts = new SMTPSwitchReborn.VirtualArray(accountManager.accounts, Components.interfaces.nsIMsgAccount); // accounts list
	var account = null;
	var accountIdentities = null;
	var accountIdentity = null;

	for ( var idx = 0 ; idx < accounts.count() ; idx++ ) { // for each account of accounts list
		account = accounts.queryAt(idx); // gets account object

		if ( account.defaultIdentity != null ) { // if account has a default identity (= skips "Local Folder")
			accountIdentities = new SMTPSwitchReborn.VirtualArray(account.identities, Components.interfaces.nsIMsgIdentity); // gets identities

			for ( var jdx = 0 ; jdx < accountIdentities.count() ; jdx++ ) {
				accountIdentity = accountIdentities.queryAt(jdx); // gets identity object
				identityNameObj[accountIdentity.key] = accountIdentity.identityName;
			}
		}
	}

	return identityNameObj;
};

/**
 * Updates SMTP servers list
 */
SMTPSwitchReborn.updateSMTPServersList = function ()
{
    var smtpService = Components.classes["@mozilla.org/messengercompose/smtp;1"]
						.getService(Components.interfaces.nsISmtpService);

	var smtpSwitchRebornMenuPopUp = document.getElementById("smtpSwitchRebornMenuPopUp"); // menu of toolbar button

	if ( smtpSwitchRebornMenuPopUp != null ) { // if toolbar button is displayed
		// detection of default SMTP server
		var defaultSMTPServer = smtpService.defaultServer;

		this.localConsole.logInformation("updateSMTPServersList", "Default SMTP server is " + defaultSMTPServer.key);		

		// detection of current identity
		var msgIdentityMenuList = document.getElementById("msgIdentity");

		if ( msgIdentityMenuList != null ) { // (JSCTSABT)
			var accountId = null;

			// for TB >= v38 msgIdentityMenuList.value holds identity name (ex.: 'john doe <jdoe@nowhere.com>')
			// for TB < v38 msgIdentityMenuList.value holds identity id (ex.: 'id2')
			if ( SMTPSwitchReborn.isTBVersionEqualOrHigherThan("38.0a1") ) { // if TB version is posterior or equal to v38
				for ( var idx in this.identityNameObj ) {
					if ( (this.identityNameObj[idx] == msgIdentityMenuList.value) && (accountId == null) ) {
						accountId = idx;
					}
				}
			}
			else {
				accountId = msgIdentityMenuList.value;
			}

			this.localConsole.logInformation("updateSMTPServersList", "Current identity is " + accountId);		
		}

		// detection of smtp server for current identity
		var accountManager = Components.classes["@mozilla.org/messenger/account-manager;1"]
								.getService(Components.interfaces.nsIMsgAccountManager);

		var accounts = new SMTPSwitchReborn.VirtualArray(accountManager.accounts, Components.interfaces.nsIMsgAccount); // accounts list
		var account = null;
		var accountIdentities = null;
		var accountIdentity = null;

		var identitySMTPServer = null;

		for ( var idx = 0 ; (idx < accounts.count()) && ( identitySMTPServer == null ) ; idx++ ) { // for each account of accounts list
			account = accounts.queryAt(idx); // gets account object

			if ( account.defaultIdentity != null ) { // if account has a default identity (= skips "Local Folder")
				accountIdentities = new SMTPSwitchReborn.VirtualArray(account.identities, Components.interfaces.nsIMsgIdentity); // gets identities

				for ( var jdx = 0 ; (jdx < accountIdentities.count()) && ( identitySMTPServer == null ) ; jdx++ ) {
					accountIdentity = accountIdentities.queryAt(jdx); // gets identity object

					if ( accountIdentity.key == accountId ) {
						identitySMTPServer = accountIdentity.smtpServerKey;
					}
				}
			}
		}

		if ( identitySMTPServer != null ) {
			this.localConsole.logInformation("updateSMTPServersList", "Current identity SMTP server is " + identitySMTPServer);		
		}
		else {
			this.localConsole.logInformation("updateSMTPServersList", "SMTP server of current identity is global default SMTP server");		
		}

		// insertion of menu items (one for each smtp server)
		var smtpServers = ( this.isTBVersionEqualOrHigherThan("22.0a1") ) ?
							smtpService.servers
							:
							smtpService.smtpServers;

		var newMenuItem = null;

		var smtpServer = null;
		var smtpServerLabel = "";

		var identitySMTPServerMenuItem = null;
		var defaultSMTPServerMenuItem = null;

		while ( smtpServers.hasMoreElements() ) { // for each smtp server
			smtpServer = smtpServers.getNext().QueryInterface(Components.interfaces.nsISmtpServer); // gets smtp server

			newMenuItem = document.createElement("menuitem"); // creates a new menuitem

			smtpServerLabel = ( (smtpServer.description != null) && (smtpServer.description != "") && !this.displayServerAddresses ) ?
								smtpServer.description
								:
								smtpServer.displayname;

			if ( smtpServer.key == defaultSMTPServer.key ) { // if current smtp server is global default smtp server
				smtpServerLabel += " [#]";
			}

			newMenuItem.setAttribute('id', "smtpSwitchRebornSMTPServer-" + smtpServer.key); // sets menuitem label
			newMenuItem.setAttribute('type', "radio"); // sets menuitem label
			newMenuItem.setAttribute('label', smtpServerLabel); // sets menuitem label
			newMenuItem.tooltipText = smtpServer.serverURI + ":" + smtpServer.port;
			newMenuItem.addEventListener("command", function() { SMTPSwitchReborn.setSMTPServer(this.id); }, false); // sets menuitem command

			if ( smtpServer.key == identitySMTPServer ) { // if current smtp server is identity smtp server
				identitySMTPServerMenuItem = newMenuItem.id; // backups current menuitem id
			}
			if ( smtpServer.key == defaultSMTPServer.key ) { // if current smtp server is global default smtp server
				defaultSMTPServerMenuItem = newMenuItem.id; // backups current menuitem id
			}

			smtpSwitchRebornMenuPopUp.appendChild(newMenuItem);
		}

		var defaultSMTPServerMenuItem2 = null;

		// ticking of default smtp server for current identity
		defaultSMTPServerMenuItem2 = ( identitySMTPServerMenuItem != null ) ?
										document.getElementById(identitySMTPServerMenuItem)
										:
										document.getElementById(defaultSMTPServerMenuItem);

		if ( defaultSMTPServerMenuItem2 != null ) { // (JSCTSABT)
			defaultSMTPServerMenuItem2.setAttribute('checked', "true"); // sets menuitem label
		}

		// insertion of 'settings' menu item
		var separatorMenuItem = document.createElement("menuseparator"); // creates a new menuitem
		separatorMenuItem.setAttribute('id', "smtpSwitchRebornSMTPServer-separator");

		smtpSwitchRebornMenuPopUp.appendChild(separatorMenuItem);

		newMenuItem = document.createElement("menuitem"); // creates a new menuitem
		
		newMenuItem.setAttribute('id', "smtpSwitchRebornSMTPServer-settings"); // sets menuitem label
		newMenuItem.setAttribute('label', this.stringBundle.getString('settingsMenuItemLabel')); // sets menuitem label
		newMenuItem.tooltipText = this.stringBundle.getString('settingsMenuItemTooltipText'); // sets tooltip text
		newMenuItem.addEventListener("command", function() { SMTPSwitchReborn.openSettingsDialog(); }, false); // sets menuitem command
		smtpSwitchRebornMenuPopUp.appendChild(newMenuItem);
	}
	else {
		this.localConsole.logWarning("updateSMTPServersList", "smtpSwitchRebornMenuPopUp has not been found");		
	}
};

/**
 * Updates SMTP servers list
 */
SMTPSwitchReborn.clearSMTPServersList = function ()
{
	var smtpSwitchRebornMenuPopUp = document.getElementById("smtpSwitchRebornMenuPopUp"); // menu of toolbar button

	if ( smtpSwitchRebornMenuPopUp != null ) { // if toolbar button is displayed
		// clearance of existing items
		var xulElement = smtpSwitchRebornMenuPopUp.lastChild; // initializes xul element to last child
		var xulElement2 = null;
		var xulEltId = "";

		// '^': start of string
		// 'smtpSwitchRebornSMTPServer-': litteral string "smtpSwitchRebornSMTPServer-"
		// '.+': any other string
		// '$': end of string
		var menuItemIdStr = new RegExp("^smtpSwitchRebornSMTPServer-.+$");

		// search of any menupopup of native 'Send Mail' button
		while ( xulElement != null ) { // while there is a previous element
			xulEltId = xulElement.id;
			xulElement2 = xulElement.previousSibling; // gets previous element

			if ( ((xulElement.nodeName === "menuitem") || (xulElement.nodeName === "menuseparator")) && xulEltId.match(menuItemIdStr) ) { // if element is a menuitem
				smtpSwitchRebornMenuPopUp.removeChild(document.getElementById(xulEltId));
			}

			xulElement = xulElement2; // gets previous element
		}
	}
};

/**
 * Finishes add-on execution
 */
SMTPSwitchReborn.setSMTPServer = function (selId)
{
	var smtpServerId = selId.substring(selId.indexOf("-") + 1);

    var smtpService = Components.classes["@mozilla.org/messengercompose/smtp;1"]
						.getService(Components.interfaces.nsISmtpService);

	if ( smtpServerId != smtpService.defaultServer.key ) {
		var newDefaultSMTPServer = smtpService.getServerByKey(smtpServerId);

		if ( newDefaultSMTPServer != null ) {
			smtpService.defaultServer = newDefaultSMTPServer;

			this.localConsole.logInformation("setSMTPServer", "Default SMTP server has been changed to " + newDefaultSMTPServer.displayname + " (" + newDefaultSMTPServer.key + ")");
		}
		else {
			this.localConsole.logError("setSMTPServer", "Default SMTP server has not been found (" + smtpServerId + ")");
		}
	}
	else {
		this.localConsole.logInformation("setSMTPServer", "Default SMTP server has not been changed (" + smtpServerId + ")");
	}
};

/**
 * Opens settings dialog
 */
SMTPSwitchReborn.openSettingsDialog = function ()
{
	this.doNotSend = true;

	window.openDialog('chrome://SMTPSwitchReborn/content/options.xul', '', 'chrome,titlebar,toolbar,centerscreen,modal,dialog=yes', this.id);
};

/**
 * Sends message
 *
 * @param msgType (nsIMsgCompDeliverMode) message type while sending
 */
SMTPSwitchReborn.sendMessage = function (msgType)
{
	if ( !this.doNotSend ) {
		SMTPSwitchReborn.DefaultGenericSendMessage(msgType);
	}

	this.doNotSend = false;
};

/**
 * Overrides default behaviour when sending message
 *
 * @param msgType (nsIMsgCompDeliverMode) message type while sending
 */
SMTPSwitchReborn.DefaultGenericSendMessage = GenericSendMessage;

GenericSendMessage = function (msgType)
{
	switch ( msgType ) {
		case nsIMsgCompDeliverMode.Now: // (0)
		case nsIMsgCompDeliverMode.Later: // (1)
		case nsIMsgCompDeliverMode.Background: // (8)
		case nsIMsgCompDeliverMode.SaveAsDraft: // (4)
		case nsIMsgCompDeliverMode.SaveAsTemplate: // (5)
		case nsIMsgCompDeliverMode.AutoSaveAsDraft: // (7)
			SMTPSwitchReborn.sendMessage(msgType);
			break;
		//case nsIMsgCompDeliverMode.Save: // (2)
		//case nsIMsgCompDeliverMode.SaveAs: // (3)
		//case nsIMsgCompDeliverMode.SendUnsent: // (6)
		default: // all other ways of "sending a message"
			SMTPSwitchReborn.DefaultGenericSendMessage(msgType);
			break;
	}
};


// window.addEventListener("load", function() { SMTPSwitchReborn.composerOverlayStartup(); }, false);
window.addEventListener("compose-window-init", function() { SMTPSwitchReborn.composerOverlayStartup(); }, true);
window.addEventListener("unload", function() { SMTPSwitchReborn.shutdown(); }, false);
