var stringBundle;

var bakAutoDetectionAtStart;
var bakAutoDetectionAtCompose;
var bakDetectionRules;
var bakDisplayServerAddresses;
var bakDebugMode;
var bakDebugModeToFile;
var bakDebugFileName;

var smtpServersNameList;

var selectedDetectionRuleIdx;
var detectionRulesListBoxChanged;

function sendMessageToConsole(source, message, msgType)
{
	msgType = ( typeof(msgType) !== "undefined" ) ? msgType : "information";

	var prefs = Components.classes["@mozilla.org/preferences-service;1"]
					.getService(Components.interfaces.nsIPrefService)
					.getBranch("extensions." + SMTPSwitchReborn.addOnName + ".");
	prefs.QueryInterface(Components.interfaces.nsIPrefBranch);

	SMTPSwitchReborn.debugMode = prefs.getBoolPref("debugMode");
	SMTPSwitchReborn.debugModeToFile = prefs.getBoolPref("debugModeToFile");
	SMTPSwitchReborn.debugFileName = unescape(prefs.getCharPref("debugFileName")); // decodes string

	switch ( msgType ) {
		case "warning":
			SMTPSwitchReborn.localConsole.logWarning(source, message);
			break;
		case "error":
			SMTPSwitchReborn.localConsole.logError(source, message);
			break;
		case "information":
		default:
			SMTPSwitchReborn.localConsole.logInformation(source, message);
			break;
	}
}

function initializeSettings()
{
	// initializations
	initializeSMTPServersList();
	stringBundle = document.getElementById("smtpSwitchRebornOptionsStringBundle");

	// tab 'accounts smtp settings'
	initializeAccountsSMTPSettings();

	// tab 'auto detection'
	bakAutoDetectionAtStart = document.getElementById("autoDetectionAtStart").value;
	bakAutoDetectionAtCompose = document.getElementById("autoDetectionAtCompose").value;
	bakDetectionRules = document.getElementById("detectionRules").value;

	detectionRulesListBoxChanged = false;
	resetSMTPServerMenuList();
	initializeDetectionRulesList();
	initializeDetectionRuleInputBoxes();

	selectedDetectionRuleIdx = -1;

	// tab 'miscellaneous' > group 'settings'
	bakDisplayServerAddresses = document.getElementById("displayServerAddresses").value;

	// tab 'miscellaneous' > group 'help and support'
	bakDebugMode = document.getElementById("debugMode").value;
	bakDebugModeToFile = document.getElementById("debugModeToFile").value;
	bakDebugFileName = unescape(document.getElementById("debugFileName").value); // decodes string
	document.getElementById("debugModeToFileTextBox").value = bakDebugFileName;

	onClickDebugModeCheckBox(bakDebugMode);

	//
	sendMessageToConsole("options.initializeSettings", "Settings have been initialized");
}

function saveSettings()
{
	// tab 'auto detection'
	var detectionRulesStr = "";
	var detectionRulesListBox = document.getElementById("detectionRulesListBox");
	var detectionRulesArray = new Array();

	if ( (detectionRulesListBox != null) && detectionRulesListBoxChanged ) { // if any change has been made to detection rules list
		// creation of detection rules string
		for ( var idx = 0 ; idx < detectionRulesListBox.getRowCount() ; idx++ ) { // for each detection rule of detection rules listbox
			detectionRulesArray.push(new SMTPSwitchReborn.DetectionRule(
				detectionRulesListBox.getItemAtIndex(idx).children[0].getAttribute('label'),
				detectionRulesListBox.getItemAtIndex(idx).children[1].getAttribute('label'),
				detectionRulesListBox.getItemAtIndex(idx).children[2].getAttribute('value'),
				detectionRulesListBox.getItemAtIndex(idx).children[3].children[0].checked));
		}
		detectionRulesStr = SMTPSwitchReborn.JSONstringify(detectionRulesArray, SMTPSwitchReborn.preProcessSaveSettings, new Array("startIP", "endIP", "smtpId"));

		sendMessageToConsole("options.saveSettings", "Detection rules list has been modified: " + detectionRulesStr);

		document.getElementById("detectionRules").value = detectionRulesStr; // saves detection rules list
	}
	else {
		sendMessageToConsole("options.saveSettings", "Detection rules list has not been modified");
	}

	// tab 'miscellaneous' > group 'help and support'
	if ( document.getElementById("debugModeToFileTextBox").value == "" ) {
		document.getElementById("debugModeToFile").value = false;
	}
	document.getElementById("debugFileName").value = escape(document.getElementById("debugModeToFileTextBox").value); // encodes string

	//
	sendMessageToConsole("options.saveSettings", "Settings have been saved");
}

function resetSettings()
{
	// tab 'auto detection'
	document.getElementById("autoDetectionAtStart").value = bakAutoDetectionAtStart;
	document.getElementById("autoDetectionAtCompose").value = bakAutoDetectionAtCompose;
	document.getElementById("detectionRules").value = bakDetectionRules;

	detectionRulesListBoxChanged = false;
	initializeDetectionRulesList();

	selectedDetectionRuleIdx = -1;

	// tab 'settings' > group 'settings'
	document.getElementById("displayServerAddresses").value = bakDisplayServerAddresses;
	
	// tab 'miscellaneous' > group 'help and support'
	document.getElementById("debugMode").value = bakDebugMode;
	document.getElementById("debugModeToFile").value = bakDebugModeToFile;
	document.getElementById("debugModeToFileTextBox").value = bakDebugFileName;

	onClickDebugModeCheckBox(bakDebugMode);

	//
	sendMessageToConsole("options.resetSettings", "Settings have been resetted");
}

function initializeSMTPServersList()
{
    var smtpService = Components.classes["@mozilla.org/messengercompose/smtp;1"]
						.getService(Components.interfaces.nsISmtpService);

	var smtpServers = ( SMTPSwitchReborn.isTBVersionEqualOrHigherThan("22.0a1") ) ?
						smtpService.servers
						:
						smtpService.smtpServers;

	var smtpServer = null;
	var smtpServerLabel = "";

	smtpServersNameList = {};

	while ( smtpServers.hasMoreElements() ) { // for each smtp server
		smtpServer = smtpServers.getNext().QueryInterface(Components.interfaces.nsISmtpServer); // gets smtp server

		smtpServerLabel = ( (smtpServer.description != null) && (smtpServer.description != "") ) ?
							smtpServer.description + " - " + smtpServer.hostname
							:
							( (smtpServer.username != null) && (smtpServer.username != "") ) ?
								smtpServer.username + " - " + smtpServer.hostname
								:
								smtpServer.hostname;

		smtpServersNameList[smtpServer.key] = smtpServerLabel;
	}
}

// accounts smtp settings
function initializeAccountsSMTPSettings()
{
	// building of existing accounts list
	var smtpSettingsListBox = document.getElementById("smtpSettingsListBox");

	if ( smtpSettingsListBox != null ) { // (JSCTSABT)
		// clearance of default subjects list
		for ( var idx = smtpSettingsListBox.getRowCount() - 1 ; idx >= 0 ; idx-- ) { // for each account smtp settings of the listbox (reverse order)
			smtpSettingsListBox.removeItemAt(idx); // removes last item
		}

		var smtpSettingsCount = 0;

		// filling of string list
		var accountManager = Components.classes["@mozilla.org/messenger/account-manager;1"]
								.getService(Components.interfaces.nsIMsgAccountManager);

		var accounts = new SMTPSwitchReborn.VirtualArray(accountManager.accounts, Components.interfaces.nsIMsgAccount); // gets accounts list
		var account = null;
		var accountServer = null; // related server
		var accountIdentities = null;
		var accountIdentity = null;

		var newRow = null;
		var newCheckBox = null;
		var newCell = null;

		var smtpServer = "";
		var smtpServerLabel = "";
		var smtpServerTooltipText = "";
		var defaultSMTPServer = false;

		for ( var idx = 0 ; idx < accounts.count() ; idx++ ) { // for each account of accounts list
			account = accounts.queryAt(idx); // gets account
			accountServer = account.incomingServer; // gets related server

			if ( account.defaultIdentity != null ) { // if account has a default identity (= skips "Local Folder")
				accountIdentities = new SMTPSwitchReborn.VirtualArray(account.identities, Components.interfaces.nsIMsgIdentity); // gets identities

				for ( var jdx = 0 ; jdx < accountIdentities.count() ; jdx++ ) { // for each identity of current account
					accountIdentity = accountIdentities.queryAt(jdx); // gets identity object

					newRow = document.createElement('listitem');
					newRow.setAttribute('id', accountIdentity.key); // sets id
					newRow.setAttribute('allowevents', true); // allows to transfer events to embedded objects
					if ( accountIdentity.key != account.defaultIdentity.key ) { // if current identity is a secondary identity (= not default identity of current account)
						newRow.style.setProperty("font-style", "italic", null);
					}

					smtpServer = accountIdentity.smtpServerKey;
					smtpServerLabel = ( smtpServer != null ) ? smtpServersNameList[smtpServer] : stringBundle.getString('smtpServerUndefined');
					smtpServerTooltipText = ( smtpServer != null ) ? smtpServer : "";

					defaultSMTPServer = ( smtpServer != null );

					// 'defaultSMTPServer' checkbox
					newCheckBox = document.createElement('checkbox');
					newCheckBox.setAttribute('checked', defaultSMTPServer);
					newRow.appendChild(newCheckBox); // adds 'defaultSMTPServer' value

					// account id
					newCell = document.createElement('listcell');
					newCell.setAttribute('label', accountIdentity.key);
					newRow.appendChild(newCell); // adds account id

					// account name
					newCell = document.createElement('listcell');
					newCell.setAttribute('label', accountServer.prettyName + " : " + accountIdentity.identityName);
					newRow.appendChild(newCell); // adds account label

					// SMTP server
					newCell = document.createElement('listcell');
					newCell.setAttribute('label', smtpServerLabel);
					newCell.setAttribute('tooltiptext', smtpServerTooltipText);
					newRow.appendChild(newCell); // adds default subject

					smtpSettingsListBox.appendChild(newRow); // adds new individual counter to counters listbox

					smtpSettingsCount++;
				}
			}
		}

		sendMessageToConsole("options.initializeAccountsSMTPSettings", smtpSettingsCount + " accounts SMTP settings have been loaded");
	}

	sendMessageToConsole("options.initializeAccountsSMTPSettings", "Accounts SMTP settings have been initialized");
}

function onClickTickAllButton()
{
	var detectionRulesListBox = document.getElementById("smtpSettingsListBox");

	for ( var idx = 0 ; idx < detectionRulesListBox.getRowCount() ; idx++ ) { // for each file of listbox
		detectionRulesListBox.getItemAtIndex(idx).children[0].setAttribute('checked', true);
	}
}

function onClickUntickAllButton()
{
	var detectionRulesListBox = document.getElementById("smtpSettingsListBox");

	for ( var idx = 0 ; idx < detectionRulesListBox.getRowCount() ; idx++ ) { // for each file of listbox
		detectionRulesListBox.getItemAtIndex(idx).children[0].setAttribute('checked', false);
	}
}

function onClickSetSMTPServerButton()
{
	var smtpSettingsListBox = document.getElementById("smtpSettingsListBox");
	var detectionRulesListBox = document.getElementById("smtpSettingsListBox");

	if ( smtpSettingsListBox != null ) { // (JSCTSABT)
		var accountManager = Components.classes["@mozilla.org/messenger/account-manager;1"]
								.getService(Components.interfaces.nsIMsgAccountManager);

		var accounts = new SMTPSwitchReborn.VirtualArray(accountManager.accounts, Components.interfaces.nsIMsgAccount); // gets accounts list
		var account = null;
		var accountIdentities = null;
		var accountIdentity = null;

		var changeFlg = false;
		var smtpFound = false;
		var idLabel = "";

		for ( var idx = 0 ; idx < accounts.count() ; idx++ ) { // for each account of accounts list
			account = accounts.queryAt(idx); // gets account

			if ( account.defaultIdentity != null ) { // if account has a default identity (= skips "Local Folder")
				accountIdentities = new SMTPSwitchReborn.VirtualArray(account.identities, Components.interfaces.nsIMsgIdentity); // gets identities

				for ( var jdx = 0 ; jdx < accountIdentities.count() ; jdx++ ) { // for each identity of current account
					accountIdentity = accountIdentities.queryAt(jdx); // gets identity object

					smtpFound = false;

					for ( var kdx = 0 ; (kdx < detectionRulesListBox.getRowCount()) & !smtpFound ; kdx++ ) { // for each file of listbox
						idLabel = detectionRulesListBox.getItemAtIndex(kdx).children[1].getAttribute('label');

						if ( idLabel == accountIdentity.key ) {
							smtpFound = true;
							changeFlg = true;

							if ( detectionRulesListBox.getItemAtIndex(kdx).children[0].getAttribute('checked') === "true" ) {
								accountIdentity.smtpServerKey = "";

								sendMessageToConsole("options.onClickSetSMTPServerButton", "SMTP server of account " + idLabel + " has been set to 'default'");
							}
						}
					}
				}
			}
		}

		if ( changeFlg ) {
			initializeAccountsSMTPSettings();
		}
	}

}

function onClickSMTPSettingsHelpButton()
{
	SMTPSwitchReborn.displayMessageDialog(stringBundle.getString('smtpSettingsHelp'));
}

// 'auto detection' functions
function onClickAutoDetectionAtStartCheckBox(state)
{
}

function onClickAutoDetectionAtComposeCheckBox(state)
{
}

function resetSMTPServerMenuList()
{
	clearSMTPServerMenuList();
	fillSMTPServerMenuList();
}

function clearSMTPServerMenuList()
{
	var smtpServerMenuPopUp = document.getElementById("smtpServerMenuPopUp");

	if ( smtpServerMenuPopUp != null ) { // (JSCTSABT)
		// clearance of existing items
		while ( smtpServerMenuPopUp.firstChild ) {
			smtpServerMenuPopUp.removeChild(smtpServerMenuPopUp.firstChild);
		}

		// add of a first empty item (to avoid graphical artefacts when choosing an actual smtp server in the menulist)
		// this only useful in case the menulist is not editable
		var emptyMenuItem = document.createElement("menuitem"); // creates a new menuitem
		emptyMenuItem.setAttribute('label', ""); // sets empty string as menuitem label
		smtpServerMenuPopUp.appendChild(emptyMenuItem); // adds empty item into menulist
	}
}

function fillSMTPServerMenuList()
{
    var smtpService = Components.classes["@mozilla.org/messengercompose/smtp;1"]
						.getService(Components.interfaces.nsISmtpService);

	var detectionRulesListBox = document.getElementById("detectionRulesListBox");
	var smtpServerMenuList = document.getElementById("smtpServerMenuList");
	var smtpServerMenuPopUp = document.getElementById("smtpServerMenuPopUp");

	if ( (detectionRulesListBox != null) && (smtpServerMenuList != null) && (smtpServerMenuPopUp != null) ) { // (JSCTSABT)
		var newMenuItem = null;

		var smtpServer = null;
		var smtpServerLabel = "";

		for ( var smtpServerKey in smtpServersNameList ) {
			smtpServer = smtpService.getServerByKey(smtpServerKey); // gets smtp server

			smtpServerLabel = smtpServersNameList[smtpServerKey];

			if ( smtpServer != null ) { // (JSCTSABT) if server is not null
				newMenuItem = document.createElement("menuitem"); // creates a new menuitem

				if ( smtpServerKey == smtpService.defaultServer.key ) { // if current smtp server is global default smtp server
					smtpServerLabel += " [#]";
				}

				newMenuItem.setAttribute('value', smtpServerKey); // sets menuitem value (usefull for storing settings)
				newMenuItem.setAttribute('label', smtpServerLabel); // sets menuitem label
				newMenuItem.tooltipText = smtpServer.serverURI + ":" + smtpServer.port;

				smtpServerMenuPopUp.appendChild(newMenuItem);
			}
			else {
				sendMessageToConsole("options.fillSMTPServerMenuList", "SMTP Server id '" + smtpServerKey + "' (" + smtpServerLabel + ") cannot be found", "error");
			}
		}
	}
}

function checkIPAddress(ipAddress)
{
	var okFlag = true;

	// '^': start of string
	// 
	// '$': end of string
	// var reIPAddress = new RegExp("^(?:(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.){3}?(?:25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$", "i");
	var reIPAddress = new RegExp("^(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$", "i");

	okFlag = ( ipAddress.match(reIPAddress) != null );

	if ( !okFlag ) {
		SMTPSwitchReborn.displayAlertDialog(stringBundle.getString('ipAddressCheckSta') + " " + ipAddress + " " + stringBundle.getString('ipAddressCheckEnd'));

		sendMessageToConsole("options.checkIPAddress", "IP address '" + ipAddress + "' is invalid", "warning");
	}

	return okFlag;
}

function checkIPAddressesInterval(startIP, endIP)
{
	var okFlag = true;

	// '^': start of string
	// 
	// '$': end of string
	var reIPAddress = new RegExp("^(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)$", "i");

	var startIPParts = startIP.match(reIPAddress);
	var endIPParts = endIP.match(reIPAddress);

	if ( (startIPParts != null) && (endIPParts != null) ) {
		for ( var idx = 1 ; (idx <= 4) && okFlag ; idx++ ) {
			if ( parseInt(startIPParts[idx]) > parseInt(endIPParts[idx]) ) {
				SMTPSwitchReborn.displayAlertDialog(stringBundle.getString('ipAddressesIntervalCheckSta') + idx + " " + stringBundle.getString('ipAddressesIntervalCheckEnd'));
				okFlag = false;
			}
		}
	}

	return okFlag;
}

function checkIPAddressesIntervalDoublon(startIP, endIP, selectedDetectionRuleIdx)
{
	var detectionRulesListBox = document.getElementById("detectionRulesListBox");
	var smtpServerMenuList = document.getElementById("smtpServerMenuList");
	var okFlag = true;

	// search of new detection rule into current detection rules list
	for ( var idx = 0 ; (idx < detectionRulesListBox.getRowCount()) && okFlag ; idx++ ) { // for each detection rule of detection rules listbox
		if ( idx != selectedDetectionRuleIdx ) {
			if ( (startIP == detectionRulesListBox.getItemAtIndex(idx).children[0].getAttribute('label'))
				 || (endIP == detectionRulesListBox.getItemAtIndex(idx).children[1].getAttribute('label')) )
			{ // if detection rules listbox already contains current detection rule
				okFlag = false;
			}

			if ( !okFlag ) {
				SMTPSwitchReborn.displayAlertDialog(stringBundle.getString('detectionRuleDoublonSta')
													+ "[" + startIP + ";" + endIP + "] => " + smtpServersNameList[smtpServerMenuList.value]
													+ stringBundle.getString('detectionRuleDoublonMid')
													+ "[" + detectionRulesListBox.getItemAtIndex(idx).children[0].getAttribute('label') + ";" + detectionRulesListBox.getItemAtIndex(idx).children[1].getAttribute('label') + "] " + " => " + detectionRulesListBox.getItemAtIndex(idx).children[2].getAttribute('label')
													+ stringBundle.getString('detectionRuleDoublonEnd'));

				sendMessageToConsole("options.checkIPAddressesIntervalDoublon", "Detection rule '[" + startIP + ";" + endIP + "]' is in conflict", "warning");
			}
		}
	}

	return okFlag;
}

function addDetectionRule(startIP, endIP, smtpId, smtpName, onOff, init)
{
	var detectionRulesListBox = document.getElementById("detectionRulesListBox");

	// add of detection rule
	if ( detectionRulesListBox != null ) { // (JSCTSABT)
		if ( startIP != "" ) {
			var okFlag = true;

			if ( endIP == "" ) {
				endIP = startIP;
			}

			okFlag = okFlag && checkIPAddress(startIP); // checks start IP address
			okFlag = okFlag && checkIPAddress(endIP); // checks end IP address

			okFlag = okFlag && checkIPAddressesInterval(startIP, endIP); // checks interval

			okFlag = okFlag && checkIPAddressesIntervalDoublon(startIP, endIP, -1); // checks doublon

			if ( okFlag ) { // if detection rule is valid
				// insertion of new detection rule in listbox
				var newRow = null;
				var newCell = null;
				var newCheckBox = null;

				newRow = document.createElement('listitem'); // adds detection rule to detection rules listbox
				newRow.setAttribute('id', "detectionRuleRow-" + detectionRulesListBox.getRowCount()); // sets id
				newRow.setAttribute('allowevents', true); // allows to transfer events to embedded objects

				// start IP
				newCell = document.createElement('listcell');
				newCell.setAttribute('label', startIP);
				newRow.appendChild(newCell); // adds start ip

				// end IP
				newCell = document.createElement('listcell');
				newCell.setAttribute('label', endIP);
				newRow.appendChild(newCell); // adds end ip

				// smtp server
				newCell = document.createElement('listcell');
				newCell.setAttribute('value', smtpId);
				newCell.setAttribute('label', smtpName);
				newRow.appendChild(newCell); // adds smtp server

				// 'enabled' checkbox
				newCell = document.createElement('listcell');
				newCheckBox = document.createElement('checkbox');
				newCheckBox.setAttribute('id', "enableDetectionRuleCheckBox-" + detectionRulesListBox.getRowCount()); // sets id
				newCheckBox.setAttribute('checked', onOff);
				newCheckBox.addEventListener("command", function() { onSelectEnableDetectionRuleCheckBox(this.id); }, false); // sets related action

				newCell.appendChild(newCheckBox);
				newRow.appendChild(newCell); // adds 'enabled' value

				detectionRulesListBox.appendChild(newRow); // adds char detection rule to detection rules listbox

				onSelectEnableDetectionRuleCheckBox(newCheckBox.id);

				// update of GUI
				var selIdx = detectionRulesListBox.getRowCount() - 1;
				detectionRulesListBox.ensureIndexIsVisible(selIdx); // ensures item is visible

				if ( !init ) {
					detectionRulesListBox.selectedIndex = selIdx;
				}

				sendMessageToConsole("options.addDetectionRule", "Detection rule '[" + startIP + ";" + endIP + "] => " + smtpId + "' has been added");
			}
		}
	}
}

function initializeDetectionRulesList()
{
	var detectionRulesListBox = document.getElementById("detectionRulesListBox");

	// clearance of detection rules list
	if ( detectionRulesListBox != null ) { // (JSCTSABT)
		for ( var idx = detectionRulesListBox.getRowCount() - 1 ; idx >= 0 ; idx-- ) { // for each detection rule of the listbox (reverse order)
			detectionRulesListBox.removeItemAt(idx); // removes last item
		}
	}

	// extraction of detection rules list
	var detectionRulesArray = null;

	if ( (bakDetectionRules == null) || (bakDetectionRules == "") ) { // if detection rules list is empty
		detectionRulesArray = new Array();
	}
	else {
		detectionRulesArray = SMTPSwitchReborn.JSONparse(bakDetectionRules, SMTPSwitchReborn.preProcessLoadSettings, new Array("startIP", "endIP", "smtpId"));
	}

	// filling of detection rules list
    var smtpService = Components.classes["@mozilla.org/messengercompose/smtp;1"]
						.getService(Components.interfaces.nsISmtpService);

	var smtpServerMenuPopUp = document.getElementById("smtpServerMenuPopUp");

	if ( smtpServerMenuPopUp != null ) { // (JSCTSABT)
		var smtpServers = null;
		var newMenuItem = null;

		var smtpServer = null;
		var smtpServerLabel = "";

		var found = false;

		for ( var idx = 0 ; idx < detectionRulesArray.length ; idx++ ) { // for each detection rule
			smtpServers = ( SMTPSwitchReborn.isTBVersionEqualOrHigherThan("22.0a1") ) ?
							smtpService.servers
							:
							smtpService.smtpServers;

			smtpServerLabel = "";

			found = false;

			while ( (smtpServers.hasMoreElements()) && !found ) { // for each smtp server
				smtpServer = smtpServers.getNext().QueryInterface(Components.interfaces.nsISmtpServer); // gets smtp server

				if ( smtpServer.key == detectionRulesArray[idx].smtpId ) {
					if ( smtpServersNameList[smtpServer.key] != null ) {
						smtpServerLabel = smtpServersNameList[smtpServer.key];
					}
					else {
						sendMessageToConsole("options.initializeDetectionRulesList", "SMTP server name of id '" + smtpServer.key + "' cannot be found", "error");
					}

					found = true;
				}
			}

			addDetectionRule(detectionRulesArray[idx].startIP, detectionRulesArray[idx].endIP, detectionRulesArray[idx].smtpId, smtpServerLabel, detectionRulesArray[idx].on, true);
		}
	}

	sendMessageToConsole("options.initializeDetectionRulesList", idx + " detection rules have been loaded");
}

function onClickDetectionRuleAddButton()
{
	var startIPTextBox = document.getElementById("startIPTextBox");
	var endIPTextBox = document.getElementById("endIPTextBox");
	var smtpServerMenuList = document.getElementById("smtpServerMenuList");

	if ( (startIPTextBox != null) && (endIPTextBox != null) && (smtpServerMenuList != null) ) { // if textboxes are not empty
		var smtpId = smtpServerMenuList.value;
		addDetectionRule(startIPTextBox.value, endIPTextBox.value, smtpId, smtpServersNameList[smtpId], true, false);

		detectionRulesListBoxChanged = true;

		startIPTextBox.focus(); // brings back focus to input textbox
	}
}

function onClickDetectionRuleModifyButton()
{
	var startIPTextBox = document.getElementById("startIPTextBox");
	var endIPTextBox = document.getElementById("endIPTextBox");
	var smtpServerMenuList = document.getElementById("smtpServerMenuList");

	var detectionRulesListBox = document.getElementById("detectionRulesListBox");

	if ( (startIPTextBox != null) && (endIPTextBox != null) && (smtpServerMenuList != null) && (detectionRulesListBox != null) ) { // (JSCTSABT)
		var startIP = startIPTextBox.value;
		var endIP = endIPTextBox.value;
		var smtpId = smtpServerMenuList.value;

		if ( startIP != "" ) {
			var okFlag = true;

			if ( endIP == "" ) {
				endIP = startIP;
			}

			okFlag = okFlag && checkIPAddress(startIP); // checks start IP address
			okFlag = okFlag && checkIPAddress(endIP); // checks end IP address

			okFlag = okFlag && checkIPAddressesInterval(startIP, endIP); // checks interval

			okFlag = okFlag && checkIPAddressesIntervalDoublon(startIP, endIP, selectedDetectionRuleIdx); // checks doublon

			if ( okFlag ) { // if detection rule is valid
				// update of detection rule
				detectionRulesListBox.getItemAtIndex(selectedDetectionRuleIdx).children[0].setAttribute('label', startIP);
				detectionRulesListBox.getItemAtIndex(selectedDetectionRuleIdx).children[1].setAttribute('label', endIP);
				detectionRulesListBox.getItemAtIndex(selectedDetectionRuleIdx).children[2].setAttribute('value', smtpId);
				detectionRulesListBox.getItemAtIndex(selectedDetectionRuleIdx).children[2].setAttribute('label', smtpServersNameList[smtpId]);

				// update of GUI
				detectionRulesListBox.ensureIndexIsVisible(selectedDetectionRuleIdx); // ensures item is visible
				detectionRulesListBox.selectedIndex = selectedDetectionRuleIdx;

				detectionRulesListBoxChanged = true;

				sendMessageToConsole("options.onClickDetectionRuleModifyButton", "Detection rule '[" + startIP + ";" + endIP + "] => " + smtpId + "' has been modified");
			}
		}

		startIPTextBox.focus(); // brings back focus to input textbox
	}
}

function onClickDetectionRuleUpButton()
{
	var detectionRulesListBox = document.getElementById("detectionRulesListBox");

	if ( detectionRulesListBox != null ) { // (JSCTSABT)
		if ( (selectedDetectionRuleIdx != -1) && (selectedDetectionRuleIdx > 0) ) { // if an item is selected and is not the first one
			swapDetectionRuleButton(detectionRulesListBox, selectedDetectionRuleIdx, selectedDetectionRuleIdx - 1, "up");
		}
	}
}

function onClickDetectionRuleDownButton()
{
	var detectionRulesListBox = document.getElementById("detectionRulesListBox");

	if ( detectionRulesListBox != null ) { // (JSCTSABT)
		if ( (selectedDetectionRuleIdx != -1) && (selectedDetectionRuleIdx < detectionRulesListBox.getRowCount() - 1) ) { // if an item is selected and is not the last one
			swapDetectionRuleButton(detectionRulesListBox, selectedDetectionRuleIdx, selectedDetectionRuleIdx + 1, "down");
		}
	}
}

function swapDetectionRuleButton(detectionRulesListBox, detectionRule1Idx, detectionRule2Idx, msg)
{
	// update of listbox
	var row1 = detectionRulesListBox.getItemAtIndex(detectionRule1Idx);
	var row2 = detectionRulesListBox.getItemAtIndex(detectionRule2Idx);

	// saving of moved item
	var tmpStartIP = row1.children[0].getAttribute('label');
	var tmpEndIP = row1.children[1].getAttribute('label');
	var tmpSMTPId = row1.children[2].getAttribute('value');
	var tmpSMTPName = row1.children[2].getAttribute('label');
	var tmpOn = row1.children[3].children[0].checked;

	// crushing of moved item with other item
	row1.children[0].setAttribute('label', row2.children[0].getAttribute('label'));
	row1.children[1].setAttribute('label', row2.children[1].getAttribute('label'));
	row1.children[2].setAttribute('value', row2.children[2].getAttribute('value'));
	row1.children[2].setAttribute('label', row2.children[2].getAttribute('label'));
	row1.children[3].children[0].checked = row2.children[3].children[0].checked;

	onSelectEnableDetectionRuleCheckBox("enableDetectionRuleCheckBox-" + detectionRule1Idx);

	// restoration of saved item
	row2.children[0].setAttribute('label', tmpStartIP);
	row2.children[1].setAttribute('label', tmpEndIP);
	row2.children[2].setAttribute('value', tmpSMTPId);
	row2.children[2].setAttribute('label', tmpSMTPName);
	row2.children[3].children[0].checked = tmpOn;

	onSelectEnableDetectionRuleCheckBox("enableDetectionRuleCheckBox-" + detectionRule2Idx);

	detectionRulesListBox.ensureIndexIsVisible(detectionRule2Idx); // ensures item is visible
	detectionRulesListBox.selectedIndex = detectionRule2Idx; // selects item

	detectionRulesListBoxChanged = true;
	// detectionRule1Idx = detectionRule2Idx;

	sendMessageToConsole("options.swapDetectionRuleButton", "Detection rule '[" + tmpStartIP + ";" + tmpEndIP + "] => " + tmpSMTPId + "' has been moved " + msg);
}

function onClickDetectionRuleRemoveButton()
{
	var startIPTextBox = document.getElementById("startIPTextBox");
	var endIPTextBox = document.getElementById("endIPTextBox");
	var smtpServerMenuList = document.getElementById("smtpServerMenuList");

	var detectionRulesListBox = document.getElementById("detectionRulesListBox");

	if ( (startIPTextBox != null) && (endIPTextBox != null) && (smtpServerMenuList != null) && (detectionRulesListBox != null) ) { // if textboxes are not empty
		var selIdx = detectionRulesListBox.selectedIndex;

		if ( selIdx != -1 ) { // if an item is selected
			var startIP = detectionRulesListBox.getItemAtIndex(selIdx).children[0].getAttribute('label');
			var endIP = detectionRulesListBox.getItemAtIndex(selIdx).children[1].getAttribute('label');
			var smtpId = detectionRulesListBox.getItemAtIndex(selIdx).children[2].getAttribute('value');

			// removal of item
			detectionRulesListBox.removeItemAt(selIdx);

			// selection of one of other left items
			if ( detectionRulesListBox.getRowCount() > 0 ) { // if there is one item left
				if ( selIdx == detectionRulesListBox.getRowCount() ) { // if removed item was the last one in the list
					selIdx--; // chooses the current last item (was formerly the penultimate one)
				}

				detectionRulesListBox.selectedIndex = selIdx; // selects it
			}
			else {
				initializeDetectionRuleInputBoxes();
			}

			sendMessageToConsole("options.onClickDetectionRuleRemoveButton", "Detection rule '[" + startIP + ";" + endIP + "] => " + smtpId + "' has been removed");

			detectionRulesListBoxChanged = true;
		}
	}
}

function onSelectEnableDetectionRuleCheckBox(selId)
{
	var enabledCheckBox = document.getElementById(selId);

	var rowId = "detectionRuleRow-" + selId.substring(selId.indexOf("-") + 1);
	var detectionRuleRow = document.getElementById(rowId);

	if ( (enabledCheckBox != null) && (detectionRuleRow != null) ) { // (JSCTSABT)
		detectionRuleRow.children[0].setAttribute('disabled', !enabledCheckBox.checked);
		detectionRuleRow.children[1].setAttribute('disabled', !enabledCheckBox.checked);
		detectionRuleRow.children[2].setAttribute('disabled', !enabledCheckBox.checked);

		detectionRulesListBoxChanged = true;
	}
}

function onSelectDetectionRule(selIdx)
{
	var detectionRulesListBox = document.getElementById("detectionRulesListBox");
	var startIPTextBox = document.getElementById("startIPTextBox");
	var endIPTextBox = document.getElementById("endIPTextBox");
	var smtpServerMenuList = document.getElementById("smtpServerMenuList");

	if ( (detectionRulesListBox != null) && (startIPTextBox != null) && (endIPTextBox != null) && (smtpServerMenuList != null) ) { // (JSCTSABT)
		var selRow = detectionRulesListBox.selectedItem;

		if ( selRow != null ) { // if an item is selected
			// copy of selected detection rule details into input textboxes and checkboxes
			startIPTextBox.value = selRow.children[0].getAttribute('label');
			endIPTextBox.value = selRow.children[1].getAttribute('label');
			smtpServerMenuList.value = selRow.children[2].getAttribute('value');

			selectedDetectionRuleIdx = detectionRulesListBox.selectedIndex;
		}
		else { // if no item is selected
			initializeDetectionRuleInputBoxes();
		}

		detectionRulesListBox.focus(); // brings back focus to listbox
	}
}

function initializeDetectionRuleInputBoxes()
{
	var startIPTextBox = document.getElementById("startIPTextBox");
	var endIPTextBox = document.getElementById("endIPTextBox");
	var smtpServerMenuList = document.getElementById("smtpServerMenuList");

	if ( (startIPTextBox != null) && (endIPTextBox != null) && (smtpServerMenuList != null) ) { // if textboxes are not empty
		document.getElementById("startIPTextBox").value = ""; // clears input textbox
		document.getElementById("endIPTextBox").value = ""; // clears input textbox

		document.getElementById("smtpServerMenuList").selectedIndex = 0; // resets menulist
	}
}

function onClickAutomaticDetectionHelpButton()
{
	SMTPSwitchReborn.displayMessageDialog(stringBundle.getString('automaticDetectionHelp'));
}

// 'miscellaneous' functions
function onClickDebugModeCheckBox(state)
{
	document.getElementById("debugModeToFileCheckBox").disabled = !state;
	if ( !state ) {
		document.getElementById("debugModeToFileTextBox").disabled = true;
		document.getElementById("debugModeToFileButton").disabled = true;
	}
	else {
		document.getElementById("debugModeToFileTextBox").disabled = !document.getElementById("debugModeToFileCheckBox").checked;
		document.getElementById("debugModeToFileButton").disabled = !document.getElementById("debugModeToFileCheckBox").checked;
	}
}

function onClickDebugModeToFileCheckBox(state)
{
	document.getElementById("debugModeToFileTextBox").disabled = !state;
	document.getElementById("debugModeToFileButton").disabled = !state;
}

function onClickDebugModeToFileButton()
{
	var filePicker = Components.classes["@mozilla.org/filepicker;1"]
						.createInstance(Components.interfaces.nsIFilePicker);
	var homeDir = null;
	var defaultName = "";

	var debugFileName = document.getElementById("debugModeToFileTextBox").value;

	if ( debugFileName != "" ) {
		try {
			homeDir = SMTPSwitchReborn.VirtualLocalFile();
			homeDir.initWithPath(debugFileName);
			defaultName = homeDir.leafName; // should be called before 'homeDir = homeDir.parent;'
			homeDir = homeDir.parent;
		}
		catch ( err ) {
			debugFileName = "";
		}
	}

	if ( debugFileName == "" ) {
		homeDir = Components.classes["@mozilla.org/file/directory_service;1"]
					.getService(Components.interfaces.nsIProperties)
					.get("Home", Components.interfaces.nsIFile);

		defaultName = SMTPSwitchReborn.temporaryRoot + "_logfile.log";
	}

	filePicker.appendFilter("Log files", "*.log;*.txt");
	filePicker.init(window, stringBundle.getString('chooseDebugLogFile'), Components.interfaces.nsIFilePicker.modeSave);
	filePicker.displayDirectory = homeDir;
	filePicker.defaultString = defaultName;

	// nsIFilePicker.show() is deprecated for TB >= v17
	// nsIFilePicker.open() has been introduced in TB v17 but behaves awkwardly for TB v17.* (callback function has to be a component with a method named 'done', exception message is: [Exception... "'Javascript component does not have a method named: "done"' when calling method: nsIFilePickerShownCallback::done" nsresult: "0805700300 (NS_ERROR_XPC_JSOBJECT_HAS_NO_FUNCTION_NAMED)" location: "<unknown>" data: no]
	if ( SMTPSwitchReborn.isTBVersionEqualOrHigherThan("18.0a1") ) {
		filePicker.open(function() {
			onClickDebugModeToFileButton2(filePicker);
		});
	}
	else {
		var filePickerReturnVal = filePicker.show();

		if ( (filePickerReturnVal == Components.interfaces.nsIFilePicker.returnOK) || (filePickerReturnVal == Components.interfaces.nsIFilePicker.returnReplace) ) {
			onClickDebugModeToFileButton2(filePicker);
		}
	}
}

function onClickDebugModeToFileButton2(filePicker)
{
	if ( filePicker.file != null ) {
		document.getElementById("debugModeToFileTextBox").value = filePicker.file.path;
	}
}

function onClickResetDefaultsButton()
{
	if ( SMTPSwitchReborn.displayConfirmDialog(stringBundle.getString('beforeResetDefaults')) ) { // displays a confirmation dialog, and if 'ok' is selected
		var prefs = Components.classes["@mozilla.org/preferences-service;1"]
						.getService(Components.interfaces.nsIPrefService)
						.getBranch(null);

		prefs.deleteBranch("extensions." + SMTPSwitchReborn.addOnName);

		SMTPSwitchReborn.displayAlertDialog(stringBundle.getString('afterResetDefaults'));

		window.close();
	}
}

function onClickExportSettingsButton()
{
	var filePicker = Components.classes["@mozilla.org/filepicker;1"]
						.createInstance(Components.interfaces.nsIFilePicker);
	var homeDir = null;
	var defaultName = SMTPSwitchReborn.temporaryRoot + "_settings.txt";

	homeDir = Components.classes["@mozilla.org/file/directory_service;1"]
				.getService(Components.interfaces.nsIProperties)
				.get("Home", Components.interfaces.nsIFile);

	filePicker.appendFilter("Settings files", "*.txt");
	filePicker.init(window, stringBundle.getString('chooseSettingsFile'), Components.interfaces.nsIFilePicker.modeSave);
	filePicker.displayDirectory = homeDir;
	filePicker.defaultString = defaultName;

	// nsIFilePicker.show() is deprecated for TB >= v17
	// nsIFilePicker.open() has been introduced in TB v17 but behaves awkwardly for TB v17.* (callback function has to be a component with a method named 'done', exception message is: [Exception... "'Javascript component does not have a method named: "done"' when calling method: nsIFilePickerShownCallback::done" nsresult: "0805700300 (NS_ERROR_XPC_JSOBJECT_HAS_NO_FUNCTION_NAMED)" location: "<unknown>" data: no]
	if ( SMTPSwitchReborn.isTBVersionEqualOrHigherThan("18.0a1") ) {
		filePicker.open(function() {
			onClickExportSettingsButton2(filePicker);
		});
	}
	else {
		var filePickerReturnVal = filePicker.show();

		if ( (filePickerReturnVal == Components.interfaces.nsIFilePicker.returnOK) || (filePickerReturnVal == Components.interfaces.nsIFilePicker.returnReplace) ) {
			onClickExportSettingsButton2(filePicker);
		}
	}
}

function onClickExportSettingsButton2(filePicker)
{
	if ( filePicker.file != null ) { // if a file has been chosen
		var prefs = Components.classes["@mozilla.org/preferences-service;1"]
					.getService(Components.interfaces.nsIPrefService)
					.getBranch("extensions." + SMTPSwitchReborn.addOnName + ".");
		var branch = prefs.QueryInterface(Components.interfaces.nsIPrefBranch);
		var settings = branch.getChildList("", {});

		var settingName = "";
		var settingVal = null;

		try {
			// check of existence of settings file
			var settingsFile = SMTPSwitchReborn.VirtualLocalFile();
			settingsFile.initWithPath(filePicker.file.path);

			if ( !settingsFile.exists() ) {
				settingsFile.create(Components.interfaces.nsIFile.NORMAL_FILE_TYPE, 0664);
			}

			// writing of settings into file
			var settingsFileStream = Components.classes["@mozilla.org/network/file-output-stream;1"]
									.createInstance(Components.interfaces.nsIFileOutputStream);

			settingsFileStream.init(settingsFile, SMTPSwitchReborn.PR_WRONLY | SMTPSwitchReborn.PR_TRUNCATE, 0664, 0);

			var converter = Components.classes["@mozilla.org/intl/converter-output-stream;1"]
							.createInstance(Components.interfaces.nsIConverterOutputStream);
			converter.init(settingsFileStream, "UTF-8", 0, 0);

			for ( var settingId in settings ) { // for each settings
				settingName = settings[settingId];
				
				switch ( branch.getPrefType(settingName) ) {
					// PREF_STRING = 32
					// PREF_INT = 64
					// PREF_BOOL = 128
					// PREF_INVALID = ?
					case 32: settingVal = "'" + prefs.getCharPref(settingName) + "'"; break;
					case 64: settingVal = prefs.getIntPref(settingName); break;
					case 128: settingVal = prefs.getBoolPref(settingName); break;
					default: settingVal = null;
				}

				if ( settingVal != null ) {
					if ( SMTPSwitchReborn.exportSettingsMode == "PREFS" ) {
						converter.writeString("user_pref(\"extensions." + SMTPSwitchReborn.addOnName + "." + settingName + "\", " + settingVal + ");\n");
					}
					else {
						converter.writeString(settingName + "=" + settingVal + "\n");
					}
				}
				else {
					sendMessageToConsole("options.onClickExportSettingsButton2", "Type of setting '" + settingName + "' = '" + branch.getPrefType(settingName) + "' is not handled", "error");
				}
			}

			converter.close(); // closes settingsFileStream too
		}
		catch ( err ) {
			sendMessageToConsole("options.onClickExportSettingsButton2", "Error while writing setting '" + settingsName + "' to file (" + err + ")", "error");
		}
	}
}
